import 'package:flutter/material.dart';

class ColorsPage extends StatelessWidget {
  final List<MapEntry<String, Color>> colors;

  const ColorsPage({
    required this.colors,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (context, constraints) {
        return GridView.builder(
          itemCount: colors.length,
          gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
            mainAxisSpacing: 8,
            crossAxisSpacing: 8,
            crossAxisCount: _getRowCount(constraints.maxWidth),
          ),
          itemBuilder: (context, index) {
            final color = colors[index].value;
            final luminance = color.computeLuminance();

            return Container(
              color: color,
              padding: const EdgeInsets.all(8),
              child: Text(
                colors[index].key,
                style: TextStyle(
                  color: luminance > 0.5 ? Colors.black87 : Colors.white,
                ),
              ),
            );
          },
        );
      },
    );
  }

  int _getRowCount(double width) {
    if (width <= 1000) {
      return 2;
    } else if (width <= 2000) {
      return 4;
    } else {
      return 6;
    }
  }
}
